/*
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999-2001 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Struts", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 */


package org.apache.struts.action;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * The <code>newTcr</code> method of a subclass of this class 
 * returns an instance of the <code>Tcr</code> subclass.
 * Please see <a onclick="window.open('../../../../../class-diagram.html', 'class_diagram', 'height=710,width=725,scrollbars=yes,resizable=yes');return false;" href="n/a">the UML class diagram</a>
 * and <a onclick="window.open('../../../../../sequence-diagram.html', 'sequence_diagram', 'height=710,width=725,scrollbars=yes,resizable=yes');return false;" href="n/a">the UML sequence diagram</a>
 * for details on how the subclasses of <code>Tcr</code>, 
 * <code>TcrForm</code>, and <code>TcrAction</code> classes
 * interact with each other to implement the wait page functionality.
 *
 * @author Alec Missine
 */
abstract public class TcrAction extends Action {

	protected abstract Tcr newTcr( TcrForm form );

    public ActionForward perform(
				ActionMapping mapping,
				ActionForm form,
				HttpServletRequest request,
				HttpServletResponse response )
		throws IOException, ServletException 
	{
		TcrForm tcrForm = (TcrForm)form;	// this action's counterpart form
		HttpSession session = request.getSession();	

		// If processing of the form has already been started,
		//	check the background thread's progress.
		Tcr tcr = (Tcr)session.getAttribute( "tcr" );
		if( tcr != null )
			return scavenge( tcr, session, tcrForm, mapping );

		// Start processing of the form in the background thread.
		tcr = newTcr( tcrForm );
		tcr.setPath( mapping.getPath() );
		tcr.setFormName( mapping.getName() );
		tcr.start();
		session.setAttribute( "tcr", tcr );
		
		// Don't return the wait page right away - instead,
		//	allow the background thread to complete
		//	before checking its progress.
		synchronized( tcr ) {
			try { tcr.wait( tcrForm.getEtcMs() ); } 
			catch( Exception e ) { e.printStackTrace(); }
		}
		return scavenge( tcr, session, tcrForm, mapping );
    }

	ActionForward scavenge( 
		Tcr tcr,
		HttpSession session, 
		TcrForm tcrForm, 
		ActionMapping mapping ) 
	{
		// Check if the background thread is still alive.
		if( tcr.isWorking() ) {		// return the wait page

			// The "progress" string will display on the wait page.
			String progress = tcrForm.getProgress();
			progress = progress == null ? "* * " : "* " + progress;
			tcrForm.setProgress( progress );

			return( mapping.findForward( "progress" ) );	// the wait page
		}
		
		// Set up the form, clear the "tcr" session attribute.
		tcr.copyTo( tcrForm );
		session.setAttribute( "tcr", null );

		// Forward control to the specified success URI.
		return (mapping.findForward("success"));
	}
}
